@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -472,6 +473,11 @@ public:
       set_data(num, den, initialized::yes);
       return *this;
    }
+   Rational& set(mpz_t&& num, mpz_t&& den)
+   {
+      set_data(num[0], den[0], initialized::yes);
+      return *this;
+   }
 
    Rational& copy_from(mpz_srcptr num_src, mpz_srcptr den_src)
    {
@@ -1348,7 +1354,7 @@ public:
    friend
    Rational&& operator>> (Rational&& a, long k)
    {
-      return std::move(a >> k);
+      return std::move(a >>= k);
    }
    friend
    Rational operator>> (const Rational& a, int k)
@@ -1990,8 +1996,8 @@ struct cmp_scalar<Rational, Rational, void>
 template <bool is_numerator, typename TPart>
 struct object_traits<RationalParticle<is_numerator, TPart>>
    : object_traits<TPart> {
-   typedef TPart proxy_for;
-   static const bool is_temporary=true, is_persistent=false;
+   using proxy_for = TPart;
+   static constexpr bool is_temporary=true, is_persistent=false;
 };
 
 template <>
@@ -2084,7 +2090,7 @@ Integer& Integer::operator= (Rational&& b)
 
 template <>
 Rational
-pow(const Rational& base, int exp);
+pow(const Rational& base, long exp);
 
 }
 namespace polymake {
