@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -105,7 +106,7 @@ UserFunctionTemplate4perl("# @category Tropical covector decomposition"
                           "# generators."
                           "# @param Matrix<TropicalNumber<Addition,Scalar>> points"
                           "# @param Matrix<TropicalNumber<Addition,Scalar>> generators"
-                          "# @return Matrix<int>. Rows correspond to points, columns to coordinates. Each entry "
+                          "# @return Matrix<Int>. Rows correspond to points, columns to coordinates. Each entry "
                           "# encodes, how many generators contain a given point in a certain coordinate."
                           "# @example"
                           "# > $generators = new Matrix<TropicalNumber<Max>>([[0,1,0],[0,0,1],[0,\"-inf\",2]]);"
@@ -119,7 +120,7 @@ UserFunctionTemplate4perl("# @category Tropical covector decomposition"
                           "# [[covectors_of_scalar_vertices]]"
                           "# @param Matrix<Scalar> points"
                           "# @param Matrix<TropicalNumber<Addition,Scalar> > generators"
-                          "# @return Matrix<int>. Rows correspond to points, columns to coordinates. Each entry "
+                          "# @return Matrix<Int>. Rows correspond to points, columns to coordinates. Each entry "
                           "# encodes, how many generators contain a given point in a certain coordinate."
                           "# @example"
                           "# > $generators = new Matrix<TropicalNumber<Max>>([[0,1,0],[0,0,1],[0,\"-inf\",2]]);"
