@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2017
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2019
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -14,14 +15,14 @@
 #-------------------------------------------------------------------------------
 
 use strict;
-use feature 'state';
 use namespaces;
+use warnings qw(FATAL void syntax misc);
+use feature 'state';
 
 package Polymake::Core::PropertyType;
 
-declare @delimiters;
-declare $nesting_level;
 declare $trusted_value;
+declare $nested_instantiation=0;
 
 sub canonical_fallback { }
 sub equal_fallback { $_[0] == $_[1] }
@@ -29,6 +30,8 @@ sub isa_fallback : method { UNIVERSAL::isa($_[1], $_[0]->pkg) }
 sub coherent_type_fallback { undef }
 sub parse_fallback : method { croak( "no string parsing routine defined for class ", $_[0]->full_name ) }
 sub toString_fallback { "$_[0]" }
+sub serialize_fallback { $_[0] }
+sub deserialize_meth : method { $_[0]->construct->($_[1]) }
 
 sub init_fallback { }
 sub performs_deduction { 0 }
@@ -45,6 +48,7 @@ use Polymake::Struct (
    [ '$application' => '#3' ],
    [ '$extension' => '$Application::extension' ],
    [ '$super' => '#4' ],
+   [ '$params' => 'undef' ],
    [ '$dimension' => '0' ],
    [ '&canonical' => '->super || \&canonical_fallback' ],          # object => void
    [ '&equal'  => '->super || \&equal_fallback' ],                 # object1, object2 => bool
@@ -55,6 +59,9 @@ use Polymake::Struct (
    [ '&toXML' => '->super' ],                                      # object, XMLwriter, opt. attributes =>
    [ '&toXMLschema' => 'undef || \&Polymake::Core::XMLwriter::type_toXMLschema' ],          # XMLwriter, opt. attributes =>
    [ '&XMLdatatype' => '->super' ],                                # => "string" referring to a XML Schema datatype or a pattern
+   [ '&serialize' => '->super || \&serialize_fallback' ],          # object, { options } => ( value, { attrs } ) or nothing
+   [ '&deserialize' => '->super || \&deserialize_meth' ],
+   [ '&JSONschema' => 'undef' ],                                   # TODO
    [ '$construct_node' => 'undef' ],                               # Overload::Node
    [ '&construct' => '\&construct_object' ],                       # args ... => object
    [ '&parse' => '->super || \&parse_fallback' ],                  # "string" => object
@@ -63,14 +70,11 @@ use Polymake::Struct (
    [ '&init' => '->super || \&init_fallback' ],
    [ '$context_pkg' => 'undef' ],
    [ '$cppoptions' => 'undef' ],
+   [ '$operators' => 'undef' ],
    [ '$help' => 'undef' ],              # InteractiveHelp (in interactive mode, when comments are supplied for user methods)
 );
 
-declare @override_methods=qw( canonical equal isa coherent_type parse toString XMLdatatype init );
-
-declare @string_ops=map { $_ => eval <<"." } qw( . cmp eq ne lt le gt ge );
-sub { \$_[2] ? "\$_[1]" $_ "\$_[0]" : "\$_[0]" $_ "\$_[1]" }
-.
+declare @override_methods=qw( canonical equal isa coherent_type serialize parse toString XMLdatatype init );
 
 ####################################################################################
 #
@@ -81,12 +85,14 @@ sub { \$_[2] ? "\$_[1]" $_ "\$_[0]" : "\$_[0]" $_ "\$_[1]" }
 sub new {
    my $self=&_new;
    my $pkg=$self->pkg;
-   Overload::learn_package_retrieval($self, \&pkg);
+   Struct::learn_package_retrieval($self, \&pkg);
    my $self_sub=sub { $self };
    define_function($pkg, "type", $self_sub, 1);
    define_function($pkg, ".type", $self_sub);
    if ($self->super) {
       $self->dimension=$self->super->dimension;
+      $self->params=$self->super->params;
+      $self->operators=$self->super->operators;
       if ($self->construct_node=$self->super->construct_node) {
          create_method_new($self);
       }
@@ -112,34 +118,32 @@ sub derived {
    my ($self)=@_;
    map { $_->type } @{mro::get_isarev($self->pkg)}
 }
+
+# perl 5.16 had a weird bug in c3 mro
+if ($] < 5.018) {
+   *derived = sub {
+      my ($self)=@_;
+      map { $_->type } grep { not /::SUPER$/ } @{mro::get_isarev($self->pkg)}
+   }
+}
 ##################################################################################
 sub add_constructor {
-   my ($self, $name, $code, $arg_types)=@_;
+   my ($self, $name, $label, $code, $arg_types)=@_;
    $self->construct_node //= do {
       create_method_new($self);
-      new Overload::Node(undef, undef, 0);
+      new_root Overload::Node;
    };
-   Overload::add_instance($self->pkg, ".$name", $code, undef, $arg_types, undef, $self->construct_node);
+   Overload::add_instance($self->pkg, ".$name", $label, $code, $arg_types, undef, $self->construct_node);
 }
 
 sub create_method_new : method {
    my ($self)=@_;
-   define_function($self->pkg, "new",
-                   $self->abstract
-                   ? sub {
-                        if (@_>1 && instanceof namespaces::TypeExpression($_[1])) {
-                           # full parameterized type: ready to use
-                           splice @_, 0, 2, @{$_[1]};
-                        } else {
-                           # bare generic name: try to create the type instance with default parameters
-                           splice @_, 0, 1, $self->pkg->typeof();
-                        }
-                        &construct_object
-                     }
-                   : sub {
-                        splice @_, 0, 1, $self;
-                        &construct_object
-                     });
+   if ($self->abstract) {
+      # bare generic name: try to create the type instance with default parameters
+      define_function($self->pkg, "new", sub { splice @_, 0, 1, $self->pkg->typeof(); &construct_object });
+   } else {
+      define_function($self->pkg, "new", sub { splice @_, 0, 1, $self; &construct_object });
+   }
 }
 
 sub construct_object : method {
@@ -184,6 +188,8 @@ sub concrete_typecheck : method {
    ? $self : undef
 }
 
+sub no_typecheck : method { croak( $_[0]->name, " inadvertently involved in overload resolution" ) }
+
 sub descend_to_generic {
    my ($self, $pkg)=@_;
    if (defined $pkg) {
@@ -197,9 +203,9 @@ sub descend_to_generic {
 ##################################################################################
 # used in overload resolution
 sub typecheck : method {
-   my ($self, $arg_list, $arg, $backtrack)=@_;
-   if (defined (my $obj_proto=Overload::fetch_type($arg))) {
-      $self->perform_typecheck->($arg_list, $obj_proto, $backtrack);
+   my ($self, $full_args, $args, $arg_index, $backtrack)=@_;
+   if (defined (my $obj_proto = Overload::fetch_type($args->[$arg_index]))) {
+      $self->perform_typecheck->($full_args, $obj_proto, $backtrack);
    }
 }
 ##################################################################################
@@ -221,32 +227,19 @@ sub add_upgrade_relations {
 #################################################################################
 # produce a name sufficient for reconstruction from a data file
 sub qualified_name {
-   my ($self, $in_app)=@_;
-   (not(defined($in_app) and $in_app==$self->application || $in_app->imported->{$self->application->name}) && $self->application->name . "::") . $self->name
-}
-#################################################################################
-sub locate_own_help_topic {
-   my ($self, $whence, $force)=@_;
-   return if !defined($self->application);
-   my $full_name=$self->full_name;
-   my $topic=$self->application->help->find($whence, $full_name) //
-             (defined($self->generic)
-              ? ($force ? croak( "internal error: asked to create a help node for a parametrized type instance" )
-                        : $self->application->help->find($whence, $self->generic->name))
-              : ($force ? $self->application->help->add([$whence, $full_name])
-                        : return));
-   if (!exists $topic->annex->{type}) {
-      weak($topic->annex->{type}=$self);
-      push @{$topic->related},
-           uniq( map { ($_, @{$_->related}) } grep { defined and $_ != $topic and !defined($_->spez_topic) } map { $_->help_topic }
-                 $whence eq "property_types" ? defined($self->super) ? $self->super : () : @{$self->super} );
-   }
-   $self->help=$topic;
+   my ($self, $in_app) = @_;
+   (not(defined($in_app) and $in_app == $self->application || $in_app->imported->{$self->application->name}) && $self->application->name . "::") .
+   $self->name
 }
 
+sub required_extensions {
+   my ($self, $include_bundled) = @_;
+   defined($self->extension) && ($include_bundled >= $self->extension->is_bundled) ? ($self->extension) : ()
+}
+#################################################################################
 sub help_topic {
-   my $self=shift;
-   $self->help // locate_own_help_topic($self, "property_types", @_);
+   my $self = shift;
+   $self->help // ($self->application // return)->help->find_type_topic($self, "property_types", @_);
 }
 #################################################################################
 package Polymake::Core::PropertyParamedType;
@@ -259,8 +252,8 @@ use Polymake::Struct (
    [ '$application' => '#1->application' ],
    [ '$extension' => '#1->extension' ],
    [ '$super' => '#2 // #1' ],
-   [ '$generic' => '#1' ],
    [ '$params' => '#3' ],
+   [ '$generic' => '#1' ],
    '@derived_abstract',
 );
 
@@ -286,10 +279,10 @@ use Polymake::Struct (
 #
 sub new {
    my $self=&_new;
-   Overload::learn_package_retrieval($self, \&PropertyType::pkg);
+   Struct::learn_package_retrieval($self, \&PropertyType::pkg);
    scan_params($self);
    unless ($self->abstract) {
-      if ($self->super && $self->super != $self->generic) {
+      if (defined($self->super) && $self->super != $self->generic) {
          # methods defined for the own generic class have precedence over those from the super class
          my $generic_index=Struct::get_field_index(\&generic);
          foreach my $method (@override_methods) {
@@ -302,8 +295,13 @@ sub new {
       define_function($self->pkg, "type", $self_sub, 1);
       define_function($self->pkg, ".type", $self_sub);
       $self->dimension=$self->super->dimension;
+      $self->operators=$self->super->operators;
       $self->construct_node=$self->super->construct_node;
+      $self->create_method_new();
       establish_inheritance($self, $self->generic, $self->super);
+      if (defined($self->generic->cppoptions)) {
+         $self->application->cpp->add_template_instance($self, $self->generic, $nested_instantiation);
+      }
       $self->init->();
    }
    $self;
@@ -356,11 +354,12 @@ sub new_generic {
    my $min_params=@$params-$n_defaults;
    my $param_index=-1;
    my @param_holders=map { new ClassTypeParam($_, $pkg, $app, ++$param_index) } @$params;
-   my $symtab=get_pkg($pkg);
+   my $symtab=get_symtab($pkg);
    if (!defined($subpkg)) {
       $self=_new_generic($self_pkg, $name, $pkg, $app, $super, \@param_holders);
       if ($super) {
          $self->dimension=$super->dimension;
+         $self->operators=$super->operators;
          if ($self->construct_node=$super->construct_node) {
             $self->create_method_new();
          }
@@ -369,14 +368,6 @@ sub new_generic {
       }
       $subpkg="props";
    }
-   define_function($symtab, "instanceof",
-                   sub {
-                      if (@_>1 && instanceof namespaces::TypeExpression($_[1])) {
-                         UNIVERSAL::isa($_[2], $_[1]->[0]->pkg);
-                      } else {
-                         UNIVERSAL::isa($_[1], $pkg);
-                      }
-                   });
    define_function($symtab, "_min_params", sub { $min_params });
 
    no strict 'refs';
@@ -424,6 +415,17 @@ sub scan_params {
    }
 }
 #################################################################################
+sub find_super_type_param {
+   my ($self, $name) = @_;
+   my @result;
+   while (defined ($self = $self->super)) {
+      if (defined($self->params) and my ($param) = grep { $_->name eq $name } @{$self->params}) {
+         @result = ($self, $param);
+      }
+   }
+   @result;
+}
+#################################################################################
 sub full_name {
    my ($self)=@_;
    if ($self->abstract ? !$self->performs_deduction : !defined($self->params)) {
@@ -446,10 +448,15 @@ sub xml_name {
 
 # produce a name sufficient for reconstruction from a data file
 sub qualified_name {
-   my ($self, $in_app)=@_;
+   my ($self, $in_app) = @_;
    $in_app //= $self->application;
    &PropertyType::qualified_name . "<" . join(", ", map { $_->qualified_name($in_app) } @{$self->params}) . ">"
 }
+
+sub required_extensions {
+   my ($self, $include_bundled) = @_;
+   map { PropertyType::required_extensions($_, $include_bundled) } $self, @{$self->params}
+}
 #################################################################################
 # Update the list of independent extensions in $_[0].
 # Both arguments may be undef (equivalent to an empty list) or single Extension objects (equivalent to a one-element list)
@@ -523,6 +530,9 @@ sub new {
    define_function($self->pkg, "type", $self_sub, 1);
    define_function($self->pkg, ".type", $self_sub);
    establish_inheritance($self, $self->generic, $self->super);
+   if (defined($self->generic->cppoptions)) {
+      $self->application->cpp->add_template_instance($self, $self->generic, $nested_instantiation);
+   }
    $self;
 }
 
@@ -550,6 +560,7 @@ use Polymake::Struct (
    [ '$super' => 'undef' ],
    [ '&abstract' => '\&extract_type' ],
    [ '&perform_typecheck' => '\&no_typecheck' ],
+   [ '$pkg' => '"UNIVERSAL"' ],
    [ '$context_pkg' => '#2' ],
    [ '$type_param_index' => '#4' ],
 );
@@ -561,9 +572,12 @@ sub extract_type : method {
    (is_object($_[1]) ? $_[1] : $_[1]->[0])->params->[$_[0]->type_param_index]
 }
 
-*concrete_type=\&PropertyParamedType::concrete_type;
+sub typecheck : method {
+   my ($self, $full_args, $args, $arg_index)=@_;
+   &extract_type->isa->($args->[$arg_index]) && $self;
+}
 
-sub no_typecheck : method { croak( $_[0]->name, " inadvertently involved in overload resolution" ) }
+*concrete_type=\&PropertyParamedType::concrete_type;
 
 sub full_name {
    my ($self)=@_;
@@ -659,7 +673,7 @@ sub typeof {
       instanceof FunctionTypeParam($_[1])
         or croak( "type_upgrade is only applicable to function type parameters" );
       state %inst_cache;
-      $inst_cache{ $_[1] } ||= &new;
+      $inst_cache{ $_[1] } //= &new;
 
    } elsif (@_==3) {
       # final typecheck clause
@@ -695,22 +709,25 @@ use Polymake::Struct (
    [ '$super' => 'undef' ],
    [ '$generic' => 'undef' ],
    [ '$params' => '[ #1 ]' ],
-   [ '$context_pkg' => '#1->context_pkg' ],
+   [ '$context_pkg' => 'undef' ],
    [ '&perform_typecheck' => '\&check_upgradable' ],
 );
 
 sub new {
    my $self=&_new;
-   $self->abstract= $self->params->[0]->abstract
-                    ? sub : method { $_[0]->params->[0]->abstract->($_[1]) }
-                    : sub : method { $_[0]->params->[0] };
+   if ($self->params->[0]->abstract) {
+      $self->abstract = sub : method { $_[0]->params->[0]->abstract->($_[1]) };
+      $self->context_pkg = $self->params->[0]->context_pkg;
+   } else {
+      $self->abstract = sub : method { $_[0]->params->[0] };
+   }
    $self;
 }
 
 sub typeof {
    @_==2 or croak( "type_upgrades_to requires exactly one type parameter" );
    state %inst_cache;
-   $inst_cache{ $_[1] } ||= &new;
+   $inst_cache{ $_[1] } //= &new;
 }
 
 sub check_upgradable : method {
@@ -725,6 +742,38 @@ sub check_upgradable : method {
 
 sub type_param_index : method { $_[0]->params->[0]->type_param_index }
 
+#################################################################################
+package Polymake::Core::PropertyType::ConvertTo;
+
+use Polymake::Struct (
+   [ '@ISA' => 'UpgradesTo' ],
+   [ new => '$' ],
+   [ '$name' => '"can_convert_to"' ],
+   [ '&perform_typecheck' => '\&no_typecheck' ],
+);
+
+sub typeof {
+   @_==2 or croak( "can_convert_to requires exactly one type parameter" );
+   state %inst_cache;
+   $inst_cache{ $_[1] } //= &new;
+}
+
+sub typecheck : method {
+   my ($self, $full_args, $args, $arg_index, $backtrack)=@_;
+   if (defined (my $target_proto=$self->abstract->($full_args))) {
+      my $given=$args->[$arg_index];
+      if ($target_proto->isa->($given)) {
+         return $self;
+      }
+      if (defined (my $converted=eval { $target_proto->construct->($given) })) {
+         push @$backtrack, sub { $args->[$arg_index]=$given };
+         $args->[$arg_index]=$converted;
+         return $self;
+      }
+   }
+   undef
+}
+
 #################################################################################
 package main;
 
@@ -733,6 +782,7 @@ sub UNIVERSAL::typeof_gen { @_==1 && $_[0] }
 
 *type_upgrades_to::=\%Polymake::Core::PropertyType::UpgradesTo::;
 *type_upgrade::=\%Polymake::Core::PropertyType::Upgrade::;
+*can_convert_to::=\%Polymake::Core::PropertyType::ConvertTo::;
 
 1
 
