@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2016
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2019
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -15,6 +16,7 @@
 
 use strict;
 use namespaces;
+use warnings qw(FATAL void syntax misc);
 
 package Polymake::Core::PropertyValue;
 
@@ -25,7 +27,12 @@ use Polymake::Struct (
    [ '$flags' => '#3' ],        # *ref flags
 );
 
-use Polymake::Enum qw( is: temporary=1024 strong_ref=1 weak_ref=2 ref=3 );
+use Polymake::Enum Flags => {
+   is_temporary => 1024,
+   is_strong_ref => 1,
+   is_weak_ref => 2,
+   is_ref => 3
+};
 
 ######################################################################
 #
@@ -34,9 +41,9 @@ use Polymake::Enum qw( is: temporary=1024 strong_ref=1 weak_ref=2 ref=3 );
 #  new PropertyValue(Property, data, [ flags ]);
 #
 sub new {
-   my $self=&_new;
-   if ($self->flags & $is_ref) {
-      weak($self->value) if $self->flags & $is_weak_ref;
+   my $self = &_new;
+   if ($self->flags & Flags::is_ref) {
+      weak($self->value) if $self->flags & Flags::is_weak_ref;
       readonly($self->value);
    } else {
       readonly_deep($self->value);
@@ -57,11 +64,11 @@ sub toString {
    defined($self->value) ? $self->property->type->toString->($self->value) : $UNDEF;
 }
 
-sub is_temporary { $_[0]->flags & $is_temporary }
+sub is_temporary { $_[0]->flags & Flags::is_temporary }
 
 sub delete_from_subobjects {
    my ($self, $trans)=@_;
-   if ($self->property->flags & $Property::is_subobject_array) {
+   if ($self->property->flags & Property::Flags::is_subobject_array) {
       delete @{$trans->subobjects}{@{$self->value}};
    }
 }
@@ -69,15 +76,37 @@ sub delete_from_subobjects {
 sub copy {
    my ($self, $parent_obj, $to_prop)=@_;
    $to_prop //= $self->property;
-   if ($self->flags & $is_ref || !defined($self->value)) {
+   if ($self->flags & Flags::is_ref || !defined($self->value)) {
       # TODO: special treatment for inner-object-tree references
       new($self, $to_prop, $self->value, $self->flags);
    } else {
       $to_prop->copy->($self->value, $parent_obj, $to_prop != $self->property);
    }
 }
+
+sub serialize {
+   my ($self, $options) = @_;
+   if ($self->property->flags & Property::Flags::is_non_storable or $self->flags & Flags::is_weak_ref) {
+      ()
+   } elsif (defined($self->value)) {
+      if (is_object($self->value)) {
+         my $type = $self->value->type;
+         my $result = $type->serialize->($self->value, $options);
+         if ($self->property->flags & Property::Flags::is_subobject or $type == $self->property->type) {
+            $result
+         } else {
+            ($result, $type->qualified_name($self->property->belongs_to->application))
+         }
+      } else {
+         $self->value
+      }
+   } else {
+      undef
+   }
+}
+
 ######################################################################
-package _::BackRefToParent;
+package Polymake::Core::PropertyValue::BackRefToParent;
 
 use Polymake::Struct (
    [ new => '$;$' ],
@@ -87,13 +116,14 @@ use Polymake::Struct (
 sub value { $_[0]->owner->parent }
 sub property { $_[0]->owner->property }
 sub is_temporary { $_[0]->owner->is_temporary }
-sub flags { $is_weak_ref }
+sub flags { Flags::is_weak_ref }
 *copy=\&new;
 
 sub toString { '@PARENT' }
+sub serialize { () }
 
 ######################################################################
-package __::Multiple;
+package Polymake::Core::PropertyValue::Multiple;
 
 use Polymake::Struct (
    [ new => '$$' ],
@@ -116,6 +146,11 @@ sub toString {
    @{$self->values} ? $self->property->type->toString->($self->value) : $PropertyValue::UNDEF;
 }
 
+sub serialize {
+   my ($self, $options) = @_;
+   [ map { $_->serialize($options) } @{$self->values} ]
+}
+
 sub flags { 0 }
 sub is_temporary { 0 }
 
@@ -126,10 +161,9 @@ sub select_now {
         or return;
    }
    if ($index && @_==3) {
-      my $scope=($_[2] //= new Scope());
-      $scope->begin_locals;
-      local_swap($self->values, 0, $index);
-      $scope->end_locals;
+      local with(($_[2] //= new Scope())->locals) {
+         local swap @{$self->values}, 0, $index;
+      }
       $index=0;
    }
    $self->values->[$index]
@@ -197,33 +231,32 @@ sub backup {
 ######################################################################
 # private:
 sub find_instance {
-   my $self=shift; shift; # parent
-   my $proto=$self->property->type;
-   my (@props, @values);
+   my $self = shift;  shift; # parent
+   my $proto = $self->property->type;
+   my (@paths, @values);
    if (is_string($_[0])) {
-      for (my $i=0; $i<$#_; ++$i) {
-         push @props, $proto->property($_[$i]);
+      for (my $i = 0; $i < $#_; ++$i) {
+         push @paths, [ $proto->encode_descending_path($_[$i]) ];
          push @values, $_[++$i];
       }
    } elsif (is_hash($_[0])) {
-      my $list=shift;
-      while (my ($prop, $value)=each %$list) {
-         push @props, $proto->property($prop);
+      my $list = shift;
+      while (my ($path, $value)=each %$list) {
+         push @paths, [ $proto->encode_descending_path($path) ];
          push @values, $value;
       }
    } else {
       croak( "PROPERTY => value or { PROPERTY => value, ... } are allowed as multiple property filter expression" );
    }
-   my $index=-1;
+   my $index = -1;
  OBJ:
    foreach my $obj (@{$self->values}) {
       ++$index;
-      my ($i, $content_index, $pv)=(0);
-      foreach my $prop (@props) {
-         defined ($content_index=$obj->dictionary->{$prop->key})  &&
-         defined ($pv=$obj->contents->[$content_index]) &&
-         !$obj->diff_properties($pv, $values[$i++])
+      for (my $i = 0; $i < @paths; ++$i) {
+         (my ($subobj, $pv) = $obj->lookup_descending_path($paths[$i])) == 2
            or next OBJ;
+         $subobj->diff_properties($pv, $values[$i])
+           and next OBJ;
       }
       return ($index, $obj);
    }
@@ -247,19 +280,20 @@ sub find_by_name {
 }
 
 sub find_or_create {
-   my ($self, $parent, $filter)=@_;
+   my ($self, $parent, $filter) = @_;
    &find_instance // do {
+      local interrupts(block);
       my $need_commit;
       if (!defined($parent->transaction)) {
-         $need_commit=1;
+         $need_commit = true;
          $parent->begin_transaction;
       }
       if (!@{$self->values}) {
          push @{$parent->contents}, $self;
          $parent->dictionary->{$self->property->key}=$#{$parent->contents};
       }
-      my $temp= @_%2 && pop;
-      my $obj=$self->property->accept->($self->property->type->pure_type->construct->(is_hash($filter) ? (%$filter) : (), @_), $parent, 0, $temp);
+      my $temp = @_ % 2 && pop;
+      my $obj = $self->property->accept->($self->property->type->pure_type->construct->(is_hash($filter) ? (%$filter) : (), @_), $parent, 0, $temp);
       ensure_unique_name($self, $obj, $temp);
       push @{$self->values}, $obj;
       if ($temp) {
@@ -274,11 +308,12 @@ sub find_or_create {
 }
 
 sub replace_or_add {
-   my $obj=pop;
-   my ($self, $parent, $filter)=@_;
-   $obj=$self->property->accept->($obj, $parent);
+   local interrupts(block);
+   my $obj = pop;
+   my ($self, $parent, $filter) = @_;
+   $obj = $self->property->accept->($obj, $parent);
    ensure_unique_name($self, $obj);
-   if (my ($index, $old)=&find_instance) {
+   if (my ($index, $old) = &find_instance) {
       splice @{$self->values}, $index, 1, $obj;
       if ($old->is_temporary) {
          $parent->transaction->forget_temporary_subobject($old);
@@ -296,7 +331,7 @@ sub replace_or_add {
 
 sub remove_instance {
    my ($self, $sub_obj)=@_;
-   my $sub_index=list_index($self->values, $sub_obj);
+   my $sub_index = list_index($self->values, $sub_obj);
    if ($sub_index>=0) {
       splice @{$self->values}, $sub_index, 1;
    } else {
